###############################################################################
#---------Programa de ejemplo para aplicación en Ing. Electrónica--------------
#-------------Informática 2020 - Facultad de Ingeniería UNaM-------------------
#
# Axel Sk.
#
#-------Detección de Temperatura de aceros Mediante Análisis de Imagen---------
############################################################################### 
import cv2
import numpy as np
from random import randint  #Para valores aleatorio en isotermas y referencias.
                            #Si el color no contrasta bien, basta con ejecutar
                            #nuevamente el código.
     
ARCHIVO = 'Tabla_sin_ref'    #Nombre de la imagen a procesar

imagen = cv2.imread(ARCHIVO)     #Para trabajar con la imagen
original = cv2.imread(ARCHIVO)   #Para visualización y comparación de resultados
hsv = cv2.cvtColor(imagen, cv2.COLOR_BGR2HSV)
 
#-----------------------------RANGOS APROXIMADOS-------------------------------
bajo_550 = np.array([1,100,70], dtype=np.uint8)
alto_550 = np.array([5,160,80], dtype=np.uint8)
#----------------------------------------------
bajo_630 = np.array([1,130,85], dtype=np.uint8)
alto_630 = np.array([3,160,95], dtype=np.uint8)
#----------------------------------------------
bajo_680 = np.array([1,160,150], dtype=np.uint8)
alto_680 = np.array([3,190,170], dtype=np.uint8)
#----------------------------------------------
bajo_740 = np.array([2,180,210], dtype=np.uint8)
alto_740 = np.array([4,200,220], dtype=np.uint8)
#----------------------------------------------
bajo_780 = np.array([1,180,225], dtype=np.uint8)
alto_780 = np.array([3,210,235], dtype=np.uint8)
#----------------------------------------------
bajo_810 = np.array([0,187,233], dtype=np.uint8)
alto_810 = np.array([3,198,242], dtype=np.uint8)
#----------------------------------------------
bajo_850 = np.array([1,180,235], dtype=np.uint8)
alto_850 = np.array([2,185,240], dtype=np.uint8)
#----------------------------------------------
bajo_900 = np.array([0,178,237], dtype=np.uint8)
alto_900 = np.array([3,185,239], dtype=np.uint8)
#----------------------------------------------
bajo_950 = np.array([10,65,235], dtype=np.uint8)
alto_950 = np.array([11,255,255], dtype=np.uint8)
#----------------------------------------------
bajo_1000 = np.array([12,170,235], dtype=np.uint8)
alto_1000 = np.array([16,200,255], dtype=np.uint8)
#----------------------------------------------
bajo_1100 = np.array([21,190,240], dtype=np.uint8)
alto_1100 = np.array([25,220,255], dtype=np.uint8)
#----------------------------------------------
bajo_1200 = np.array([23,170,240], dtype=np.uint8)
alto_1200 = np.array([25,187,255], dtype=np.uint8)
#----------------------------------------------
bajo_1300 = np.array([15,100,240], dtype=np.uint8)
alto_1300 = np.array([25,130,255], dtype=np.uint8)
#------------------------------------------------------------------------------

#---------------------Máscaras para los rangos aproximados---------------------
mask_550 = cv2.inRange(hsv, bajo_550, alto_550)
mask_630 = cv2.inRange(hsv, bajo_630, alto_630)
mask_680 = cv2.inRange(hsv, bajo_680, alto_680)
mask_740 = cv2.inRange(hsv, bajo_740, alto_740)
mask_780 = cv2.inRange(hsv, bajo_780, alto_780)
mask_810 = cv2.inRange(hsv, bajo_810, alto_810)
mask_850 = cv2.inRange(hsv, bajo_850, alto_850)
mask_900 = cv2.inRange(hsv, bajo_900, alto_900)
mask_950 = cv2.inRange(hsv, bajo_950, alto_950)
mask_1000 = cv2.inRange(hsv, bajo_1000, alto_1000)
mask_1100 = cv2.inRange(hsv, bajo_1100, alto_1100)
mask_1200 = cv2.inRange(hsv, bajo_1200, alto_1200)
mask_1300 = cv2.inRange(hsv, bajo_1300, alto_1300)
#------------------------------------------------------------------------------

datos = [mask_550, mask_630, mask_680, mask_740, mask_780, mask_810, mask_850,\
         mask_900, mask_950, mask_1000, mask_1100, mask_1200, mask_1300]
ref_temp = [550, 630, 680, 740, 780, 810, 850, 900, 950, 1000, 1100, 1200, 1300]

#-----------------------Filtrado de contornos y texto--------------------------
font = cv2.FONT_HERSHEY_SIMPLEX #Fuente utilizada para el texto en pantalla
x = 0                           #Coordenadas X e Y, posición en pantalla
y = 0
mask_total = 0                  #Máscara resultado de cada detección

for indice in datos:
    contornos, _  = cv2.findContours(indice, cv2.RETR_EXTERNAL, \
                                     cv2.CHAIN_APPROX_NONE)
    color_1 = randint(0,255)    #Colores para la tupla BGR de los contornos
    color_2 = randint(0,255)    #detectados y el texto que lo referencia.
    color_3 = randint(0,255)
    
    for c in contornos:
        area = cv2.contourArea(c)
        if area > 10:# and area < 10000:  #Si quiero puedo definir tamaño máx.
            cv2.drawContours(imagen, [c], 0, (color_1, color_2, color_3), 2, \
                             cv2.LINE_AA)
            cv2.putText(imagen,"T=" + str(ref_temp[y]) + " C",(x,y*22+20),\
                        font,0.5,(color_1, color_2, color_3),1,cv2.LINE_AA)
    y = y + 1       #Indice para las referencias y cambio de coordenadas para
                    #el texto en pantalla. Se puede mejorar mucho.

    mask_total = cv2.add(mask_total,indice) #Agrego a la máscara total la
                                            #máscara obtenida en cada iteración.
#------------------------------------------------------------------------------
#Mostrar la mascara obtenida y la imagen original para comparar
cv2.imshow('Imagen Original', original)
cv2.imshow('Mascara con detecciones', mask_total)
cv2.imshow('Imagen + Isotermas', imagen)
#------------------------------------------------------------------------------
#Mantiene abiertas la imagenes. Con la tecla "Esc" (escape) se cancela el bucle
while(1):
    tecla = cv2.waitKey(5) & 0xFF  #Espera que se presione una tecla
    if tecla == 27:
        break 
cv2.destroyAllWindows()
###############################################################################
    
